unit DynamicDslVariant;

interface

uses
  TypInfo, Variants, Classes;

type
  { the actual base class }
  TDynamicClass = class
  private type
    { support class, don't use directly }
    TDynamicVariantType = class (TPublishableVariantType)
    protected
      class function GetDynamicObject(const V: TVarData): TDynamicClass;
      function GetInstance(const V: TVarData): TObject; override;
    public
      procedure Clear(var V: TVarData); override;
      procedure Copy(var Dest: TVarData; const Source: TVarData;
        const Indirect: Boolean); override;
      procedure CastTo(var Dest: TVarData; const Source: TVarData; const AVarType: TVarType); override;
      function DoFunction(var Dest: TVarData; const V: TVarData;
        const Name: string; const Arguments: TVarDataArray): Boolean; override;
      function DoProcedure(const V: TVarData; const Name: string;
        const Arguments: TVarDataArray): Boolean; override;
    end;
  private
    theVariant: variant;
    class var theVariantType: TDynamicVariantType;
  protected
    function GetDynamicObject(const V: TVarData): TDynamicClass;
  public
    constructor Create;

    function DoFunction(var Dest: TVarData; const V: TVarData;
      const Name: string; const Arguments: TVarDataArray): Boolean; virtual;
    function DoProcedure(const V: TVarData; const Name: string;
      const Arguments: TVarDataArray): Boolean; virtual;
    function AsString: string; virtual;

    property v:  Variant read theVariant write theVariant;
    property :  Variant read theVariant write theVariant;
    property :  Variant read theVariant write theVariant;
    property :  Variant read theVariant write theVariant;
    property :  Variant read theVariant write theVariant;
    property :  Variant read theVariant write theVariant;
    property :  Variant read theVariant write theVariant;
    property &do:  Variant read theVariant write theVariant;
  end;

implementation

uses
  VarUtils, SysUtils, Windows;

type
  { Helper record for variant data }
  TDynamicVarData = packed record
    VType: TVarType;
    Reserved1, Reserved2, Reserved3: Word;
    VDateTime: TDynamicClass;
    Reserved4: DWord;
  end;

{ TDynamicClass.TDynamicVariantType nested class }

procedure TDynamicClass.TDynamicVariantType.CastTo(var Dest: TVarData;
  const Source: TVarData; const AVarType: TVarType);
//var
//  LTemp: TVarData;
begin
  if Source.VType = VarType then
    case AVarType of
      varString:
        VarDataFromLStr(Dest, AnsiString(TDynamicVarData (Source).VDateTime.AsString));
      varUString:
        VarDataFromStr(Dest, TDynamicVarData (Source).VDateTime.AsString);
      varOleStr:
        VarDataFromOleStr(Dest, TDynamicVarData (Source).VDateTime.AsString);

        //    else
//      VarDataInit(LTemp);
//      try
//        LTemp.VType := varDate;
//        LTemp.VDate := TDslDateTimeVarData (Source).VDateTime.DateTime;
//        VarDataCastTo(Dest, LTemp, AVarType);
//      finally
//        VarDataClear(LTemp);
//      end;
    end
  else
    inherited;
end;

procedure TDynamicClass.TDynamicVariantType.Clear(var V: TVarData);
begin
  V.VType := varEmpty;
  // FreeAndNil(TDynamicVarData(V).VDateTime);
end;

procedure TDynamicClass.TDynamicVariantType.Copy(var Dest: TVarData;
  const Source: TVarData; const Indirect: Boolean);
begin
  SimplisticCopy(Dest, Source, Indirect); // use standard implementaion
end;

function TDynamicClass.TDynamicVariantType.DoFunction(var Dest: TVarData;
  const V: TVarData; const Name: string;
  const Arguments: TVarDataArray): Boolean;
begin
  Result := GetDynamicObject(V).
    DoFunction (Dest, V, Name, Arguments);
end;

function TDynamicClass.TDynamicVariantType.DoProcedure(const V: TVarData;
  const Name: string; const Arguments: TVarDataArray): Boolean;
begin
  Result := GetDynamicObject (V).
    DoProcedure (V, Name, Arguments);
end;

class function TDynamicClass.TDynamicVariantType.GetDynamicObject(const V: TVarData): TDynamicClass;
begin
  Result := TDynamicVarData(V).VDateTime;
end;

function TDynamicClass.TDynamicVariantType.GetInstance(const V: TVarData): TObject;
begin
  Result := TDynamicVarData(V).VDateTime;
end;

{ TDynamicClass }

function TDynamicClass.AsString: string;
begin
  Result := ClassName;
end;

constructor TDynamicClass.Create;
begin
  // first time: class initialization
  if not Assigned(theVariantType) then
    theVariantType := TDynamicVariantType.Create;

  // variant construction
  VarClear(theVariant);
  TDynamicVarData(theVariant).VType :=
    TDynamicClass.theVariantType.VarType;
  TDynamicVarData(theVariant).VDateTime := self;
end;

function TDynamicClass.DoFunction(var Dest: TVarData; const V: TVarData;
  const Name: string; const Arguments: TVarDataArray): Boolean;
begin
  // do nothing
  Result := True;
end;

function TDynamicClass.DoProcedure(const V: TVarData; const Name: string;
  const Arguments: TVarDataArray): Boolean;
begin
  // do nothing
  Result := True;
end;

function TDynamicClass.GetDynamicObject(const V: TVarData): TDynamicClass;
begin
  Result := TDynamicVariantType.GetDynamicObject (V);
end;

//finalization
// FreeAndNil(DslDateTimeVariantType);

end.



